package de.szut.casino.blackjack;

import de.szut.casino.user.UserEntity;
import de.szut.casino.user.UserRepository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;
import java.util.Random;

@Service
public class BlackJackService {
    private final BlackJackGameRepository blackJackGameRepository;
    private final UserRepository userRepository;

    public BlackJackService(BlackJackGameRepository blackJackGameRepository, UserRepository userRepository) {
        this.blackJackGameRepository = blackJackGameRepository;
        this.userRepository = userRepository;
    }

    private final Random random = new Random();

    public BlackJackGameEntity getBlackJackGame(Long id) {
        Optional<BlackJackGameEntity> optionalBlackJackGame = blackJackGameRepository.findById(id);
        return optionalBlackJackGame.orElse(null);
    }

    @Transactional
    public BlackJackGameEntity createBlackJackGame(UserEntity user, BigDecimal betAmount) {
        BlackJackGameEntity game = new BlackJackGameEntity();
        game.setUser(user);
        game.setBet(betAmount);
        initializeDeck(game);

        for (int i = 0; i < 2; i++) {
            CardEntity playerCard = drawCardFromDeck(game);
            playerCard.setCardType(CardType.PLAYER);
            game.getPlayerCards().add(playerCard);
        }

        CardEntity dealerCard = drawCardFromDeck(game);
        dealerCard.setCardType(CardType.DEALER);
        game.getDealerCards().add(dealerCard);

        BlackJackState state = handleState(game, user);
        game.setState(state);

        userRepository.save(user);
        return blackJackGameRepository.save(game);
    }

    @Transactional
    public BlackJackGameEntity hit(BlackJackGameEntity game) {
        game = blackJackGameRepository.findById(game.getId()).orElse(game);
        
        if (game.getState() != BlackJackState.IN_PROGRESS) {
            return game;
        }
        
        CardEntity drawnCard = drawCardFromDeck(game);
        drawnCard.setCardType(CardType.PLAYER);
        game.getPlayerCards().add(drawnCard);

        game.setState(handleState(game, game.getUser()));

        if (game.getState() == BlackJackState.PLAYER_WON) {
            updateUserBalance(game, true);
        } else if (game.getState() == BlackJackState.PLAYER_LOST) {
            updateUserBalance(game, false);
        }

        return blackJackGameRepository.save(game);
    }

    @Transactional
    public BlackJackGameEntity stand(BlackJackGameEntity game) {
        game = blackJackGameRepository.findById(game.getId()).orElse(game);
        
        if (game.getState() != BlackJackState.IN_PROGRESS) {
            return game;
        }
        
        while (calculateHandValue(game.getDealerCards()) < 17) {
            CardEntity dealerCard = drawCardFromDeck(game);
            dealerCard.setCardType(CardType.DEALER);
            game.getDealerCards().add(dealerCard);
        }

        int playerValue = calculateHandValue(game.getPlayerCards());
        int dealerValue = calculateHandValue(game.getDealerCards());

        if (dealerValue > 21 || playerValue > dealerValue) {
            game.setState(BlackJackState.PLAYER_WON);
            updateUserBalance(game, true);
        } else {
            game.setState(BlackJackState.DRAW);
            updateUserBalance(game, false);
        }

        return blackJackGameRepository.save(game);
    }

    @Transactional
    private void updateUserBalance(BlackJackGameEntity game, boolean isWin) {
        UserEntity user = game.getUser();
        user = userRepository.findById(user.getId()).orElse(user);
        BigDecimal balance = user.getBalance();
        
        BigDecimal betAmount = game.getBet();
        
        if (isWin) {
            balance = balance.add(betAmount.multiply(BigDecimal.valueOf(2)));
        } 
        else if (game.getState() == BlackJackState.DRAW) {
            balance = balance.add(betAmount);
        }
        
        user.setBalance(balance);
        userRepository.save(user);
    }

    private void initializeDeck(BlackJackGameEntity game) {
        for (Suit suit : Suit.values()) {
            for (Rank rank : Rank.values()) {
                CardEntity card = new CardEntity();
                card.setGame(game);
                card.setSuit(suit);
                card.setRank(rank);
                card.setCardType(CardType.DECK);
                game.getDeck().add(card);
            }
        }

        java.util.Collections.shuffle(game.getDeck(), random);
    }

    private CardEntity drawCardFromDeck(BlackJackGameEntity game) {
        if (game.getDeck().isEmpty()) {
            throw new IllegalStateException("Deck is empty");
        }

        return game.getDeck().removeFirst();
    }

    private BlackJackState handleState(BlackJackGameEntity game, UserEntity user) {
        int playerHandValue = calculateHandValue(game.getPlayerCards());

        if (playerHandValue == 21) {
            CardEntity hole = drawCardFromDeck(game);
            hole.setCardType(CardType.DEALER);
            game.getDealerCards().add(hole);

            int dealerHandValue = calculateHandValue(game.getDealerCards());

            if (dealerHandValue == 21) {
                return BlackJackState.STANDOFF;
            } else {
                BigDecimal blackjackWinnings = game.getBet().multiply(new BigDecimal("1.5"));
                user.setBalance(user.getBalance().add(blackjackWinnings));
                return BlackJackState.PLAYER_BLACKJACK;
            }
        } else if (playerHandValue > 21) {
            user.setBalance(user.getBalance().subtract(game.getBet()));
            return BlackJackState.PLAYER_LOST;
        }

        return BlackJackState.IN_PROGRESS;
    }

    private int calculateHandValue(List<CardEntity> hand) {
        int sum = 0;
        int aceCount = 0;
        for (CardEntity card : hand) {
            sum += card.getRank().getValue();
            if (card.getRank() == Rank.ACE) {
                aceCount++;
            }
        }

        while (sum > 21 && aceCount > 0) {
            sum -= 10;
            aceCount--;
        }

        return sum;
    }
}
