import { ChangeDetectionStrategy, Component, inject, signal } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NavbarComponent } from '../../../shared/components/navbar/navbar.component';
import { Router } from '@angular/router';
import { UserService } from '../../../service/user.service';
import { PlayingCardComponent } from './components/playing-card/playing-card.component';
import { DealerHandComponent } from './components/dealer-hand/dealer-hand.component';
import { PlayerHandComponent } from './components/player-hand/player-hand.component';
import { GameControlsComponent } from './components/game-controls/game-controls.component';
import { GameInfoComponent } from './components/game-info/game-info.component';
import { Card, BlackjackGame } from './models/blackjack.model';
import { BlackjackService } from './services/blackjack.service';
import { HttpErrorResponse } from '@angular/common/http';
import { GameResultComponent } from './components/game-result/game-result.component';

@Component({
  selector: 'app-blackjack',
  standalone: true,
  imports: [
    CommonModule,
    NavbarComponent,
    PlayingCardComponent,
    DealerHandComponent,
    PlayerHandComponent,
    GameControlsComponent,
    GameInfoComponent,
    GameResultComponent,
  ],
  templateUrl: './blackjack.component.html',
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export default class BlackjackComponent {
  private router = inject(Router);
  private userService = inject(UserService);
  private blackjackService = inject(BlackjackService);

  dealerCards = signal<Card[]>([]);
  playerCards = signal<Card[]>([]);
  currentBet = signal(0);
  balance = signal(0);
  currentGameId = signal<number | undefined>(undefined);
  gameInProgress = signal(false);
  gameState = signal<string>('IN_PROGRESS');
  showGameResult = signal(false);

  constructor() {
    this.refreshUserBalance();
  }

  private refreshUserBalance(): void {
    this.userService.getCurrentUser().subscribe((user) => {
      this.balance.set(user?.balance ?? 0);
    });
  }

  private updateGameState(game: BlackjackGame) {
    console.log('Game state update:', game);
    this.currentGameId.set(game.id);
    this.currentBet.set(game.bet);
    this.gameInProgress.set(game.state === 'IN_PROGRESS');
    this.gameState.set(game.state);

    this.dealerCards.set(
      game.dealerCards.map((card, index) => ({
        ...card,
        hidden: index === 1 && game.state === 'IN_PROGRESS',
      }))
    );

    this.playerCards.set(
      game.playerCards.map((card) => ({
        ...card,
        hidden: false,
      }))
    );

    if (game.state !== 'IN_PROGRESS') {
      this.refreshUserBalance();
      setTimeout(() => {
        this.showGameResult.set(true);
      }, 1000);
    }
  }

  onNewGame(bet: number): void {
    this.blackjackService.startGame(bet).subscribe({
      next: (game) => {
        this.updateGameState(game);
        this.refreshUserBalance();
      },
      error: (error) => {
        console.error('Failed to start game:', error);
      },
    });
  }

  onHit(): void {
    if (!this.currentGameId()) return;

    this.blackjackService.hit(this.currentGameId()!).subscribe({
      next: (game) => {
        this.updateGameState(game);
      },
      error: (error) => {
        console.error('Failed to hit:', error);
        this.handleGameError(error);
      },
    });
  }

  onStand(): void {
    if (!this.currentGameId()) return;
    
    if (this.gameState() !== 'IN_PROGRESS') {
      console.log('Cannot stand: game is not in progress');
      return;
    }

    this.blackjackService.stand(this.currentGameId()!).subscribe({
      next: (game) => {
        this.updateGameState(game);
      },
      error: (error) => {
        console.error('Failed to stand:', error);
        this.handleGameError(error);
      },
    });
  }

  onCloseGameResult(): void {
    this.showGameResult.set(false);
  }

  private handleGameError(error: any): void {
    if (error instanceof HttpErrorResponse) {
      if (error.status === 400 && error.error?.error === 'Invalid state') {
        this.gameInProgress.set(false);
        
        this.refreshUserBalance();
      } 
      else if (error.status === 500) {
        console.log('Server error occurred. The game may have been updated in another session.');
        
        this.gameInProgress.set(false);
        
        this.refreshUserBalance();
        
        if (this.currentGameId()) {
          this.refreshGameState(this.currentGameId()!);
        }
      }
    }
  }

  private refreshGameState(gameId: number): void {
    this.blackjackService.getGame(gameId).subscribe({
      next: (game) => {
        this.updateGameState(game);
      },
      error: (err) => {
        console.error('Failed to refresh game state:', err);
      }
    });
  }

  leaveGame(): void {
    this.router.navigate(['/home']);
  }
}
